/****************************************************************************
**
** Copyright (C) 2018 The Qt Company Ltd and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the plugins of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or (at your option) the GNU General
** Public license version 3 or any later version approved by the KDE Free
** Qt Foundation. The licenses are as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-2.0.html and
** https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QtQml/qqmlextensionplugin.h>
#include <QtQml/qqml.h>

#include "qdeclarativeservice_p.h"
#include "qdeclarativeserviceold_p.h"

QT_BEGIN_NAMESPACE

class QServiceDeclarativeModule : public QQmlExtensionPlugin
{
    Q_OBJECT
    Q_PLUGIN_METADATA(IID "org.qt-project.Qt.QQmlExtensionInterface" FILE "serviceframework.json")

public:
    virtual void registerTypes(const char *uri)
    {
        //Q_ASSERT(QLatin1String(uri) == QLatin1String("QtServiceFramework"));
        if (QLatin1String(uri) != QLatin1String("QtServiceFramework")) {
            qWarning("!!! Using deprecated name for QtServiceFramework module.  Rename all imports to QtServiceFramework. This will break shortly. !!!");
        }

        // @uri QtServiceFramework 5.0

        qmlRegisterType<QDeclarativeServiceLoader>(uri, 5, 0, "ServiceLoader");
        qmlRegisterUncreatableType<QDeclarativeServiceDescriptor>(uri, 5, 0, "ServiceDescriptor", "ServiceDescriptor elements must be generated by a ServiceFilter.");
        qmlRegisterType<QDeclarativeServiceFilter>(uri, 5, 0, "ServiceFilter");

        // Old types, scheduled for eventual deprecation
        qmlRegisterType<QDeclarativeService>(uri, 5, 0, "Service");
        qmlRegisterType<QDeclarativeServiceList>(uri, 5, 0, "ServiceList");
    }
};

QT_END_NAMESPACE
#include "serviceframework.moc"
