/****************************************************************************
** Meta object code from reading C++ file 'qservicereply.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/serviceframework/qservicereply.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qservicereply.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QServiceReplyBase_t {
    QByteArrayData data[12];
    char stringdata0[127];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QServiceReplyBase_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QServiceReplyBase_t qt_meta_stringdata_QServiceReplyBase = {
    {
QT_MOC_LITERAL(0, 0, 17), // "QServiceReplyBase"
QT_MOC_LITERAL(1, 18, 7), // "started"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 8), // "finished"
QT_MOC_LITERAL(4, 36, 12), // "errorChanged"
QT_MOC_LITERAL(5, 49, 5), // "start"
QT_MOC_LITERAL(6, 55, 6), // "finish"
QT_MOC_LITERAL(7, 62, 8), // "setError"
QT_MOC_LITERAL(8, 71, 22), // "QServiceManager::Error"
QT_MOC_LITERAL(9, 94, 5), // "error"
QT_MOC_LITERAL(10, 100, 14), // "setProxyObject"
QT_MOC_LITERAL(11, 115, 11) // "proxyObject"

    },
    "QServiceReplyBase\0started\0\0finished\0"
    "errorChanged\0start\0finish\0setError\0"
    "QServiceManager::Error\0error\0"
    "setProxyObject\0proxyObject"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QServiceReplyBase[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,
       3,    0,   50,    2, 0x06 /* Public */,
       4,    0,   51,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   52,    2, 0x08 /* Private */,
       6,    0,   53,    2, 0x08 /* Private */,
       7,    1,   54,    2, 0x08 /* Private */,
      10,    1,   57,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, QMetaType::QObjectStar,   11,

       0        // eod
};

void QServiceReplyBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QServiceReplyBase *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->started(); break;
        case 1: _t->finished(); break;
        case 2: _t->errorChanged(); break;
        case 3: _t->start(); break;
        case 4: _t->finish(); break;
        case 5: _t->setError((*reinterpret_cast< QServiceManager::Error(*)>(_a[1]))); break;
        case 6: _t->setProxyObject((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QServiceReplyBase::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QServiceReplyBase::started)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QServiceReplyBase::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QServiceReplyBase::finished)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QServiceReplyBase::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QServiceReplyBase::errorChanged)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QServiceReplyBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QServiceReplyBase.data,
    qt_meta_data_QServiceReplyBase,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QServiceReplyBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QServiceReplyBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QServiceReplyBase.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QServiceReplyBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void QServiceReplyBase::started()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QServiceReplyBase::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QServiceReplyBase::errorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
struct qt_meta_stringdata_QServiceReply_t {
    QByteArrayData data[1];
    char stringdata0[14];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QServiceReply_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QServiceReply_t qt_meta_stringdata_QServiceReply = {
    {
QT_MOC_LITERAL(0, 0, 13) // "QServiceReply"

    },
    "QServiceReply"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QServiceReply[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QServiceReply::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QServiceReply::staticMetaObject = { {
    QMetaObject::SuperData::link<QServiceReplyBase::staticMetaObject>(),
    qt_meta_stringdata_QServiceReply.data,
    qt_meta_data_QServiceReply,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QServiceReply::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QServiceReply::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QServiceReply.stringdata0))
        return static_cast<void*>(this);
    return QServiceReplyBase::qt_metacast(_clname);
}

int QServiceReply::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QServiceReplyBase::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
