/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativenetworkinfo_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../git/src/imports/systeminfo/qdeclarativenetworkinfo_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativenetworkinfo_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QDeclarativeNetworkInfo_t {
    QByteArrayData data[92];
    char stringdata0[1768];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QDeclarativeNetworkInfo_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QDeclarativeNetworkInfo_t qt_meta_stringdata_QDeclarativeNetworkInfo = {
    {
QT_MOC_LITERAL(0, 0, 23), // "QDeclarativeNetworkInfo"
QT_MOC_LITERAL(1, 24, 39), // "monitorCurrentCellDataTechnol..."
QT_MOC_LITERAL(2, 64, 0), // ""
QT_MOC_LITERAL(3, 65, 32), // "monitorCurrentNetworkModeChanged"
QT_MOC_LITERAL(4, 98, 35), // "monitorNetworkSignalStrengthC..."
QT_MOC_LITERAL(5, 134, 35), // "monitorNetworkInterfaceCountC..."
QT_MOC_LITERAL(6, 170, 27), // "monitorNetworkStatusChanged"
QT_MOC_LITERAL(7, 198, 20), // "monitorCellIdChanged"
QT_MOC_LITERAL(8, 219, 38), // "monitorCurrentMobileCountryCo..."
QT_MOC_LITERAL(9, 258, 38), // "monitorCurrentMobileNetworkCo..."
QT_MOC_LITERAL(10, 297, 30), // "monitorLocationAreaCodeChanged"
QT_MOC_LITERAL(11, 328, 25), // "monitorNetworkNameChanged"
QT_MOC_LITERAL(12, 354, 13), // "cellIdChanged"
QT_MOC_LITERAL(13, 368, 14), // "interfaceIndex"
QT_MOC_LITERAL(14, 383, 2), // "id"
QT_MOC_LITERAL(15, 386, 32), // "currentCellDataTechnologyChanged"
QT_MOC_LITERAL(16, 419, 4), // "tech"
QT_MOC_LITERAL(17, 424, 31), // "currentMobileCountryCodeChanged"
QT_MOC_LITERAL(18, 456, 3), // "mcc"
QT_MOC_LITERAL(19, 460, 31), // "currentMobileNetworkCodeChanged"
QT_MOC_LITERAL(20, 492, 3), // "mnc"
QT_MOC_LITERAL(21, 496, 25), // "currentNetworkModeChanged"
QT_MOC_LITERAL(22, 522, 23), // "locationAreaCodeChanged"
QT_MOC_LITERAL(23, 546, 3), // "lac"
QT_MOC_LITERAL(24, 550, 28), // "networkInterfaceCountChanged"
QT_MOC_LITERAL(25, 579, 4), // "mode"
QT_MOC_LITERAL(26, 584, 5), // "count"
QT_MOC_LITERAL(27, 590, 18), // "networkNameChanged"
QT_MOC_LITERAL(28, 609, 4), // "name"
QT_MOC_LITERAL(29, 614, 28), // "networkSignalStrengthChanged"
QT_MOC_LITERAL(30, 643, 8), // "strength"
QT_MOC_LITERAL(31, 652, 20), // "networkStatusChanged"
QT_MOC_LITERAL(32, 673, 6), // "status"
QT_MOC_LITERAL(33, 680, 35), // "_q_currentCellDataTechnologyC..."
QT_MOC_LITERAL(34, 716, 9), // "interface"
QT_MOC_LITERAL(35, 726, 32), // "QNetworkInfo::CellDataTechnology"
QT_MOC_LITERAL(36, 759, 31), // "_q_networkInterfaceCountChanged"
QT_MOC_LITERAL(37, 791, 25), // "QNetworkInfo::NetworkMode"
QT_MOC_LITERAL(38, 817, 31), // "_q_networkSignalStrengthChanged"
QT_MOC_LITERAL(39, 849, 23), // "_q_networkStatusChanged"
QT_MOC_LITERAL(40, 873, 27), // "QNetworkInfo::NetworkStatus"
QT_MOC_LITERAL(41, 901, 21), // "_q_networkNameChanged"
QT_MOC_LITERAL(42, 923, 21), // "networkSignalStrength"
QT_MOC_LITERAL(43, 945, 11), // "NetworkMode"
QT_MOC_LITERAL(44, 957, 21), // "networkInterfaceCount"
QT_MOC_LITERAL(45, 979, 25), // "currentCellDataTechnology"
QT_MOC_LITERAL(46, 1005, 13), // "networkStatus"
QT_MOC_LITERAL(47, 1019, 6), // "cellId"
QT_MOC_LITERAL(48, 1026, 24), // "currentMobileCountryCode"
QT_MOC_LITERAL(49, 1051, 24), // "currentMobileNetworkCode"
QT_MOC_LITERAL(50, 1076, 16), // "locationAreaCode"
QT_MOC_LITERAL(51, 1093, 11), // "networkName"
QT_MOC_LITERAL(52, 1105, 21), // "homeMobileCountryCode"
QT_MOC_LITERAL(53, 1127, 21), // "homeMobileNetworkCode"
QT_MOC_LITERAL(54, 1149, 4), // "imsi"
QT_MOC_LITERAL(55, 1154, 10), // "macAddress"
QT_MOC_LITERAL(56, 1165, 28), // "monitorNetworkSignalStrength"
QT_MOC_LITERAL(57, 1194, 20), // "monitorNetworkStatus"
QT_MOC_LITERAL(58, 1215, 18), // "monitorNetworkName"
QT_MOC_LITERAL(59, 1234, 25), // "monitorCurrentNetworkMode"
QT_MOC_LITERAL(60, 1260, 18), // "currentNetworkMode"
QT_MOC_LITERAL(61, 1279, 28), // "monitorNetworkInterfaceCount"
QT_MOC_LITERAL(62, 1308, 32), // "monitorCurrentCellDataTechnology"
QT_MOC_LITERAL(63, 1341, 13), // "monitorCellId"
QT_MOC_LITERAL(64, 1355, 31), // "monitorCurrentMobileCountryCode"
QT_MOC_LITERAL(65, 1387, 31), // "monitorCurrentMobileNetworkCode"
QT_MOC_LITERAL(66, 1419, 23), // "monitorLocationAreaCode"
QT_MOC_LITERAL(67, 1443, 18), // "CellDataTechnology"
QT_MOC_LITERAL(68, 1462, 21), // "UnknownDataTechnology"
QT_MOC_LITERAL(69, 1484, 18), // "GprsDataTechnology"
QT_MOC_LITERAL(70, 1503, 18), // "EdgeDataTechnology"
QT_MOC_LITERAL(71, 1522, 18), // "UmtsDataTechnology"
QT_MOC_LITERAL(72, 1541, 18), // "HspaDataTechnology"
QT_MOC_LITERAL(73, 1560, 11), // "UnknownMode"
QT_MOC_LITERAL(74, 1572, 7), // "GsmMode"
QT_MOC_LITERAL(75, 1580, 8), // "CdmaMode"
QT_MOC_LITERAL(76, 1589, 9), // "WcdmaMode"
QT_MOC_LITERAL(77, 1599, 8), // "WlanMode"
QT_MOC_LITERAL(78, 1608, 12), // "EthernetMode"
QT_MOC_LITERAL(79, 1621, 13), // "BluetoothMode"
QT_MOC_LITERAL(80, 1635, 9), // "WimaxMode"
QT_MOC_LITERAL(81, 1645, 7), // "LteMode"
QT_MOC_LITERAL(82, 1653, 11), // "TdscdmaMode"
QT_MOC_LITERAL(83, 1665, 13), // "NetworkStatus"
QT_MOC_LITERAL(84, 1679, 13), // "UnknownStatus"
QT_MOC_LITERAL(85, 1693, 18), // "NoNetworkAvailable"
QT_MOC_LITERAL(86, 1712, 13), // "EmergencyOnly"
QT_MOC_LITERAL(87, 1726, 9), // "Searching"
QT_MOC_LITERAL(88, 1736, 4), // "Busy"
QT_MOC_LITERAL(89, 1741, 6), // "Denied"
QT_MOC_LITERAL(90, 1748, 11), // "HomeNetwork"
QT_MOC_LITERAL(91, 1760, 7) // "Roaming"

    },
    "QDeclarativeNetworkInfo\0"
    "monitorCurrentCellDataTechnologyChanged\0"
    "\0monitorCurrentNetworkModeChanged\0"
    "monitorNetworkSignalStrengthChanged\0"
    "monitorNetworkInterfaceCountChanged\0"
    "monitorNetworkStatusChanged\0"
    "monitorCellIdChanged\0"
    "monitorCurrentMobileCountryCodeChanged\0"
    "monitorCurrentMobileNetworkCodeChanged\0"
    "monitorLocationAreaCodeChanged\0"
    "monitorNetworkNameChanged\0cellIdChanged\0"
    "interfaceIndex\0id\0currentCellDataTechnologyChanged\0"
    "tech\0currentMobileCountryCodeChanged\0"
    "mcc\0currentMobileNetworkCodeChanged\0"
    "mnc\0currentNetworkModeChanged\0"
    "locationAreaCodeChanged\0lac\0"
    "networkInterfaceCountChanged\0mode\0"
    "count\0networkNameChanged\0name\0"
    "networkSignalStrengthChanged\0strength\0"
    "networkStatusChanged\0status\0"
    "_q_currentCellDataTechnologyChanged\0"
    "interface\0QNetworkInfo::CellDataTechnology\0"
    "_q_networkInterfaceCountChanged\0"
    "QNetworkInfo::NetworkMode\0"
    "_q_networkSignalStrengthChanged\0"
    "_q_networkStatusChanged\0"
    "QNetworkInfo::NetworkStatus\0"
    "_q_networkNameChanged\0networkSignalStrength\0"
    "NetworkMode\0networkInterfaceCount\0"
    "currentCellDataTechnology\0networkStatus\0"
    "cellId\0currentMobileCountryCode\0"
    "currentMobileNetworkCode\0locationAreaCode\0"
    "networkName\0homeMobileCountryCode\0"
    "homeMobileNetworkCode\0imsi\0macAddress\0"
    "monitorNetworkSignalStrength\0"
    "monitorNetworkStatus\0monitorNetworkName\0"
    "monitorCurrentNetworkMode\0currentNetworkMode\0"
    "monitorNetworkInterfaceCount\0"
    "monitorCurrentCellDataTechnology\0"
    "monitorCellId\0monitorCurrentMobileCountryCode\0"
    "monitorCurrentMobileNetworkCode\0"
    "monitorLocationAreaCode\0CellDataTechnology\0"
    "UnknownDataTechnology\0GprsDataTechnology\0"
    "EdgeDataTechnology\0UmtsDataTechnology\0"
    "HspaDataTechnology\0UnknownMode\0GsmMode\0"
    "CdmaMode\0WcdmaMode\0WlanMode\0EthernetMode\0"
    "BluetoothMode\0WimaxMode\0LteMode\0"
    "TdscdmaMode\0NetworkStatus\0UnknownStatus\0"
    "NoNetworkAvailable\0EmergencyOnly\0"
    "Searching\0Busy\0Denied\0HomeNetwork\0"
    "Roaming"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QDeclarativeNetworkInfo[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      38,   14, // methods
      11,  344, // properties
       3,  388, // enums/sets
       0,    0, // constructors
       0,       // flags
      20,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  204,    2, 0x06 /* Public */,
       3,    0,  205,    2, 0x06 /* Public */,
       4,    0,  206,    2, 0x06 /* Public */,
       5,    0,  207,    2, 0x06 /* Public */,
       6,    0,  208,    2, 0x06 /* Public */,
       7,    0,  209,    2, 0x06 /* Public */,
       8,    0,  210,    2, 0x06 /* Public */,
       9,    0,  211,    2, 0x06 /* Public */,
      10,    0,  212,    2, 0x06 /* Public */,
      11,    0,  213,    2, 0x06 /* Public */,
      12,    2,  214,    2, 0x06 /* Public */,
      15,    2,  219,    2, 0x06 /* Public */,
      17,    2,  224,    2, 0x06 /* Public */,
      19,    2,  229,    2, 0x06 /* Public */,
      21,    0,  234,    2, 0x06 /* Public */,
      22,    2,  235,    2, 0x06 /* Public */,
      24,    2,  240,    2, 0x06 /* Public */,
      27,    3,  245,    2, 0x06 /* Public */,
      29,    3,  252,    2, 0x06 /* Public */,
      31,    3,  259,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      33,    2,  266,    2, 0x08 /* Private */,
      36,    2,  271,    2, 0x08 /* Private */,
      38,    3,  276,    2, 0x08 /* Private */,
      39,    3,  283,    2, 0x08 /* Private */,
      41,    3,  290,    2, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      42,    2,  297,    2, 0x02 /* Public */,
      44,    1,  302,    2, 0x02 /* Public */,
      45,    1,  305,    2, 0x02 /* Public */,
      46,    2,  308,    2, 0x02 /* Public */,
      47,    1,  313,    2, 0x02 /* Public */,
      48,    1,  316,    2, 0x02 /* Public */,
      49,    1,  319,    2, 0x02 /* Public */,
      50,    1,  322,    2, 0x02 /* Public */,
      51,    2,  325,    2, 0x02 /* Public */,
      52,    1,  330,    2, 0x02 /* Public */,
      53,    1,  333,    2, 0x02 /* Public */,
      54,    1,  336,    2, 0x02 /* Public */,
      55,    2,  339,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   13,   14,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   13,   16,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   13,   18,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   13,   20,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   13,   23,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   25,   26,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::QString,   25,   13,   28,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,   25,   13,   30,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,   25,   13,   32,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, 0x80000000 | 35,   34,   16,
    QMetaType::Void, 0x80000000 | 37, QMetaType::Int,   25,   26,
    QMetaType::Void, 0x80000000 | 37, QMetaType::Int, QMetaType::Int,   25,   34,   30,
    QMetaType::Void, 0x80000000 | 37, QMetaType::Int, 0x80000000 | 40,   25,   34,   32,
    QMetaType::Void, 0x80000000 | 37, QMetaType::Int, QMetaType::QString,   25,   34,   28,

 // methods: parameters
    QMetaType::Int, 0x80000000 | 43, QMetaType::Int,   25,   34,
    QMetaType::Int, 0x80000000 | 43,   25,
    QMetaType::Int, QMetaType::Int,   34,
    QMetaType::Int, 0x80000000 | 43, QMetaType::Int,   25,   34,
    QMetaType::QString, QMetaType::Int,   34,
    QMetaType::QString, QMetaType::Int,   34,
    QMetaType::QString, QMetaType::Int,   34,
    QMetaType::QString, QMetaType::Int,   34,
    QMetaType::QString, 0x80000000 | 43, QMetaType::Int,   25,   34,
    QMetaType::QString, QMetaType::Int,   34,
    QMetaType::QString, QMetaType::Int,   34,
    QMetaType::QString, QMetaType::Int,   34,
    QMetaType::QString, 0x80000000 | 43, QMetaType::Int,   25,   34,

 // properties: name, type, flags
      56, QMetaType::Bool, 0x00495103,
      57, QMetaType::Bool, 0x00495103,
      58, QMetaType::Bool, 0x00495103,
      59, QMetaType::Bool, 0x00495103,
      60, 0x80000000 | 43, 0x00495009,
      61, QMetaType::Bool, 0x00495103,
      62, QMetaType::Bool, 0x00495103,
      63, QMetaType::Bool, 0x00495103,
      64, QMetaType::Bool, 0x00495103,
      65, QMetaType::Bool, 0x00495103,
      66, QMetaType::Bool, 0x00495103,

 // properties: notify_signal_id
       2,
       4,
       9,
       1,
      14,
       3,
       0,
       5,
       6,
       7,
       8,

 // enums: name, alias, flags, count, data
      67,   67, 0x0,    5,  403,
      43,   43, 0x0,   10,  413,
      83,   83, 0x0,    8,  433,

 // enum data: key, value
      68, uint(QDeclarativeNetworkInfo::UnknownDataTechnology),
      69, uint(QDeclarativeNetworkInfo::GprsDataTechnology),
      70, uint(QDeclarativeNetworkInfo::EdgeDataTechnology),
      71, uint(QDeclarativeNetworkInfo::UmtsDataTechnology),
      72, uint(QDeclarativeNetworkInfo::HspaDataTechnology),
      73, uint(QDeclarativeNetworkInfo::UnknownMode),
      74, uint(QDeclarativeNetworkInfo::GsmMode),
      75, uint(QDeclarativeNetworkInfo::CdmaMode),
      76, uint(QDeclarativeNetworkInfo::WcdmaMode),
      77, uint(QDeclarativeNetworkInfo::WlanMode),
      78, uint(QDeclarativeNetworkInfo::EthernetMode),
      79, uint(QDeclarativeNetworkInfo::BluetoothMode),
      80, uint(QDeclarativeNetworkInfo::WimaxMode),
      81, uint(QDeclarativeNetworkInfo::LteMode),
      82, uint(QDeclarativeNetworkInfo::TdscdmaMode),
      84, uint(QDeclarativeNetworkInfo::UnknownStatus),
      85, uint(QDeclarativeNetworkInfo::NoNetworkAvailable),
      86, uint(QDeclarativeNetworkInfo::EmergencyOnly),
      87, uint(QDeclarativeNetworkInfo::Searching),
      88, uint(QDeclarativeNetworkInfo::Busy),
      89, uint(QDeclarativeNetworkInfo::Denied),
      90, uint(QDeclarativeNetworkInfo::HomeNetwork),
      91, uint(QDeclarativeNetworkInfo::Roaming),

       0        // eod
};

void QDeclarativeNetworkInfo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QDeclarativeNetworkInfo *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->monitorCurrentCellDataTechnologyChanged(); break;
        case 1: _t->monitorCurrentNetworkModeChanged(); break;
        case 2: _t->monitorNetworkSignalStrengthChanged(); break;
        case 3: _t->monitorNetworkInterfaceCountChanged(); break;
        case 4: _t->monitorNetworkStatusChanged(); break;
        case 5: _t->monitorCellIdChanged(); break;
        case 6: _t->monitorCurrentMobileCountryCodeChanged(); break;
        case 7: _t->monitorCurrentMobileNetworkCodeChanged(); break;
        case 8: _t->monitorLocationAreaCodeChanged(); break;
        case 9: _t->monitorNetworkNameChanged(); break;
        case 10: _t->cellIdChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 11: _t->currentCellDataTechnologyChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 12: _t->currentMobileCountryCodeChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 13: _t->currentMobileNetworkCodeChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 14: _t->currentNetworkModeChanged(); break;
        case 15: _t->locationAreaCodeChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 16: _t->networkInterfaceCountChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 17: _t->networkNameChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 18: _t->networkSignalStrengthChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 19: _t->networkStatusChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 20: _t->_q_currentCellDataTechnologyChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QNetworkInfo::CellDataTechnology(*)>(_a[2]))); break;
        case 21: _t->_q_networkInterfaceCountChanged((*reinterpret_cast< QNetworkInfo::NetworkMode(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 22: _t->_q_networkSignalStrengthChanged((*reinterpret_cast< QNetworkInfo::NetworkMode(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 23: _t->_q_networkStatusChanged((*reinterpret_cast< QNetworkInfo::NetworkMode(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< QNetworkInfo::NetworkStatus(*)>(_a[3]))); break;
        case 24: _t->_q_networkNameChanged((*reinterpret_cast< QNetworkInfo::NetworkMode(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 25: { int _r = _t->networkSignalStrength((*reinterpret_cast< NetworkMode(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 26: { int _r = _t->networkInterfaceCount((*reinterpret_cast< NetworkMode(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 27: { int _r = _t->currentCellDataTechnology((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 28: { int _r = _t->networkStatus((*reinterpret_cast< NetworkMode(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 29: { QString _r = _t->cellId((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 30: { QString _r = _t->currentMobileCountryCode((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 31: { QString _r = _t->currentMobileNetworkCode((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 32: { QString _r = _t->locationAreaCode((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 33: { QString _r = _t->networkName((*reinterpret_cast< NetworkMode(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 34: { QString _r = _t->homeMobileCountryCode((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 35: { QString _r = _t->homeMobileNetworkCode((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 36: { QString _r = _t->imsi((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 37: { QString _r = _t->macAddress((*reinterpret_cast< NetworkMode(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QDeclarativeNetworkInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::monitorCurrentCellDataTechnologyChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::monitorCurrentNetworkModeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::monitorNetworkSignalStrengthChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::monitorNetworkInterfaceCountChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::monitorNetworkStatusChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::monitorCellIdChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::monitorCurrentMobileCountryCodeChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::monitorCurrentMobileNetworkCodeChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::monitorLocationAreaCodeChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::monitorNetworkNameChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)(int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::cellIdChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::currentCellDataTechnologyChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)(int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::currentMobileCountryCodeChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)(int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::currentMobileNetworkCodeChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::currentNetworkModeChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)(int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::locationAreaCodeChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::networkInterfaceCountChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)(int , int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::networkNameChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)(int , int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::networkSignalStrengthChanged)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (QDeclarativeNetworkInfo::*)(int , int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeNetworkInfo::networkStatusChanged)) {
                *result = 19;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QDeclarativeNetworkInfo *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->monitorNetworkSignalStrength(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->monitorNetworkStatus(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->monitorNetworkName(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->monitorCurrentNetworkMode(); break;
        case 4: *reinterpret_cast< NetworkMode*>(_v) = _t->currentNetworkMode(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->monitorNetworkInterfaceCount(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->monitorCurrentCellDataTechnology(); break;
        case 7: *reinterpret_cast< bool*>(_v) = _t->monitorCellId(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->monitorCurrentMobileCountryCode(); break;
        case 9: *reinterpret_cast< bool*>(_v) = _t->monitorCurrentMobileNetworkCode(); break;
        case 10: *reinterpret_cast< bool*>(_v) = _t->monitorLocationAreaCode(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QDeclarativeNetworkInfo *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setMonitorNetworkSignalStrength(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setMonitorNetworkStatus(*reinterpret_cast< bool*>(_v)); break;
        case 2: _t->setMonitorNetworkName(*reinterpret_cast< bool*>(_v)); break;
        case 3: _t->setMonitorCurrentNetworkMode(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->setMonitorNetworkInterfaceCount(*reinterpret_cast< bool*>(_v)); break;
        case 6: _t->setMonitorCurrentCellDataTechnology(*reinterpret_cast< bool*>(_v)); break;
        case 7: _t->setMonitorCellId(*reinterpret_cast< bool*>(_v)); break;
        case 8: _t->setMonitorCurrentMobileCountryCode(*reinterpret_cast< bool*>(_v)); break;
        case 9: _t->setMonitorCurrentMobileNetworkCode(*reinterpret_cast< bool*>(_v)); break;
        case 10: _t->setMonitorLocationAreaCode(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QDeclarativeNetworkInfo::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QDeclarativeNetworkInfo.data,
    qt_meta_data_QDeclarativeNetworkInfo,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QDeclarativeNetworkInfo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeNetworkInfo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QDeclarativeNetworkInfo.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativeNetworkInfo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 38)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 38;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 38)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 38;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 11;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QDeclarativeNetworkInfo::monitorCurrentCellDataTechnologyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativeNetworkInfo::monitorCurrentNetworkModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeNetworkInfo::monitorNetworkSignalStrengthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativeNetworkInfo::monitorNetworkInterfaceCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QDeclarativeNetworkInfo::monitorNetworkStatusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QDeclarativeNetworkInfo::monitorCellIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QDeclarativeNetworkInfo::monitorCurrentMobileCountryCodeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QDeclarativeNetworkInfo::monitorCurrentMobileNetworkCodeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QDeclarativeNetworkInfo::monitorLocationAreaCodeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QDeclarativeNetworkInfo::monitorNetworkNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QDeclarativeNetworkInfo::cellIdChanged(int _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QDeclarativeNetworkInfo::currentCellDataTechnologyChanged(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QDeclarativeNetworkInfo::currentMobileCountryCodeChanged(int _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QDeclarativeNetworkInfo::currentMobileNetworkCodeChanged(int _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QDeclarativeNetworkInfo::currentNetworkModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QDeclarativeNetworkInfo::locationAreaCodeChanged(int _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void QDeclarativeNetworkInfo::networkInterfaceCountChanged(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void QDeclarativeNetworkInfo::networkNameChanged(int _t1, int _t2, const QString & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void QDeclarativeNetworkInfo::networkSignalStrengthChanged(int _t1, int _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void QDeclarativeNetworkInfo::networkStatusChanged(int _t1, int _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 19, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
