/****************************************************************************
** Meta object code from reading C++ file 'qbatteryinfo_linux_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/systeminfo/linux/qbatteryinfo_linux_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qbatteryinfo_linux_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QBatteryInfoPrivate_t {
    QByteArrayData data[35];
    char stringdata0[512];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QBatteryInfoPrivate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QBatteryInfoPrivate_t qt_meta_stringdata_QBatteryInfoPrivate = {
    {
QT_MOC_LITERAL(0, 0, 19), // "QBatteryInfoPrivate"
QT_MOC_LITERAL(1, 20, 19), // "batteryCountChanged"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 5), // "count"
QT_MOC_LITERAL(4, 47, 19), // "batteryIndexChanged"
QT_MOC_LITERAL(5, 67, 12), // "batteryIndex"
QT_MOC_LITERAL(6, 80, 12), // "validChanged"
QT_MOC_LITERAL(7, 93, 7), // "isValid"
QT_MOC_LITERAL(8, 101, 18), // "chargerTypeChanged"
QT_MOC_LITERAL(9, 120, 25), // "QBatteryInfo::ChargerType"
QT_MOC_LITERAL(10, 146, 4), // "type"
QT_MOC_LITERAL(11, 151, 20), // "chargingStateChanged"
QT_MOC_LITERAL(12, 172, 27), // "QBatteryInfo::ChargingState"
QT_MOC_LITERAL(13, 200, 5), // "state"
QT_MOC_LITERAL(14, 206, 12), // "levelChanged"
QT_MOC_LITERAL(15, 219, 5), // "level"
QT_MOC_LITERAL(16, 225, 18), // "currentFlowChanged"
QT_MOC_LITERAL(17, 244, 4), // "flow"
QT_MOC_LITERAL(18, 249, 17), // "cycleCountChanged"
QT_MOC_LITERAL(19, 267, 10), // "cycleCount"
QT_MOC_LITERAL(20, 278, 24), // "remainingCapacityChanged"
QT_MOC_LITERAL(21, 303, 8), // "capacity"
QT_MOC_LITERAL(22, 312, 28), // "remainingChargingTimeChanged"
QT_MOC_LITERAL(23, 341, 7), // "seconds"
QT_MOC_LITERAL(24, 349, 14), // "voltageChanged"
QT_MOC_LITERAL(25, 364, 7), // "voltage"
QT_MOC_LITERAL(26, 372, 18), // "levelStatusChanged"
QT_MOC_LITERAL(27, 391, 25), // "QBatteryInfo::LevelStatus"
QT_MOC_LITERAL(28, 417, 11), // "levelStatus"
QT_MOC_LITERAL(29, 429, 13), // "healthChanged"
QT_MOC_LITERAL(30, 443, 20), // "QBatteryInfo::Health"
QT_MOC_LITERAL(31, 464, 6), // "health"
QT_MOC_LITERAL(32, 471, 18), // "temperatureChanged"
QT_MOC_LITERAL(33, 490, 11), // "temperature"
QT_MOC_LITERAL(34, 502, 9) // "onTimeout"

    },
    "QBatteryInfoPrivate\0batteryCountChanged\0"
    "\0count\0batteryIndexChanged\0batteryIndex\0"
    "validChanged\0isValid\0chargerTypeChanged\0"
    "QBatteryInfo::ChargerType\0type\0"
    "chargingStateChanged\0QBatteryInfo::ChargingState\0"
    "state\0levelChanged\0level\0currentFlowChanged\0"
    "flow\0cycleCountChanged\0cycleCount\0"
    "remainingCapacityChanged\0capacity\0"
    "remainingChargingTimeChanged\0seconds\0"
    "voltageChanged\0voltage\0levelStatusChanged\0"
    "QBatteryInfo::LevelStatus\0levelStatus\0"
    "healthChanged\0QBatteryInfo::Health\0"
    "health\0temperatureChanged\0temperature\0"
    "onTimeout"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QBatteryInfoPrivate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      14,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   89,    2, 0x06 /* Public */,
       4,    1,   92,    2, 0x06 /* Public */,
       6,    1,   95,    2, 0x06 /* Public */,
       8,    1,   98,    2, 0x06 /* Public */,
      11,    1,  101,    2, 0x06 /* Public */,
      14,    1,  104,    2, 0x06 /* Public */,
      16,    1,  107,    2, 0x06 /* Public */,
      18,    1,  110,    2, 0x06 /* Public */,
      20,    1,  113,    2, 0x06 /* Public */,
      22,    1,  116,    2, 0x06 /* Public */,
      24,    1,  119,    2, 0x06 /* Public */,
      26,    1,  122,    2, 0x06 /* Public */,
      29,    1,  125,    2, 0x06 /* Public */,
      32,    1,  128,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      34,    0,  131,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Void, QMetaType::Int,   17,
    QMetaType::Void, QMetaType::Int,   19,
    QMetaType::Void, QMetaType::Int,   21,
    QMetaType::Void, QMetaType::Int,   23,
    QMetaType::Void, QMetaType::Int,   25,
    QMetaType::Void, 0x80000000 | 27,   28,
    QMetaType::Void, 0x80000000 | 30,   31,
    QMetaType::Void, QMetaType::Float,   33,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void QBatteryInfoPrivate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QBatteryInfoPrivate *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->batteryCountChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->batteryIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->validChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->chargerTypeChanged((*reinterpret_cast< QBatteryInfo::ChargerType(*)>(_a[1]))); break;
        case 4: _t->chargingStateChanged((*reinterpret_cast< QBatteryInfo::ChargingState(*)>(_a[1]))); break;
        case 5: _t->levelChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->currentFlowChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->cycleCountChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->remainingCapacityChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->remainingChargingTimeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->voltageChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->levelStatusChanged((*reinterpret_cast< QBatteryInfo::LevelStatus(*)>(_a[1]))); break;
        case 12: _t->healthChanged((*reinterpret_cast< QBatteryInfo::Health(*)>(_a[1]))); break;
        case 13: _t->temperatureChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 14: _t->onTimeout(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QBatteryInfoPrivate::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfoPrivate::batteryCountChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QBatteryInfoPrivate::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfoPrivate::batteryIndexChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QBatteryInfoPrivate::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfoPrivate::validChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QBatteryInfoPrivate::*)(QBatteryInfo::ChargerType );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfoPrivate::chargerTypeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QBatteryInfoPrivate::*)(QBatteryInfo::ChargingState );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfoPrivate::chargingStateChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QBatteryInfoPrivate::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfoPrivate::levelChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QBatteryInfoPrivate::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfoPrivate::currentFlowChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QBatteryInfoPrivate::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfoPrivate::cycleCountChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QBatteryInfoPrivate::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfoPrivate::remainingCapacityChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QBatteryInfoPrivate::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfoPrivate::remainingChargingTimeChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QBatteryInfoPrivate::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfoPrivate::voltageChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QBatteryInfoPrivate::*)(QBatteryInfo::LevelStatus );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfoPrivate::levelStatusChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QBatteryInfoPrivate::*)(QBatteryInfo::Health );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfoPrivate::healthChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QBatteryInfoPrivate::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfoPrivate::temperatureChanged)) {
                *result = 13;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QBatteryInfoPrivate::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QBatteryInfoPrivate.data,
    qt_meta_data_QBatteryInfoPrivate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QBatteryInfoPrivate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QBatteryInfoPrivate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QBatteryInfoPrivate.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QBatteryInfoPrivate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void QBatteryInfoPrivate::batteryCountChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QBatteryInfoPrivate::batteryIndexChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QBatteryInfoPrivate::validChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QBatteryInfoPrivate::chargerTypeChanged(QBatteryInfo::ChargerType _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QBatteryInfoPrivate::chargingStateChanged(QBatteryInfo::ChargingState _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QBatteryInfoPrivate::levelChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QBatteryInfoPrivate::currentFlowChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QBatteryInfoPrivate::cycleCountChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QBatteryInfoPrivate::remainingCapacityChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QBatteryInfoPrivate::remainingChargingTimeChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QBatteryInfoPrivate::voltageChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QBatteryInfoPrivate::levelStatusChanged(QBatteryInfo::LevelStatus _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QBatteryInfoPrivate::healthChanged(QBatteryInfo::Health _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QBatteryInfoPrivate::temperatureChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
