/****************************************************************************
** Meta object code from reading C++ file 'qremoteserviceregister.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/serviceframework/qremoteserviceregister.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qremoteserviceregister.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QRemoteServiceRegister_t {
    QByteArrayData data[14];
    char stringdata0[243];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QRemoteServiceRegister_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QRemoteServiceRegister_t qt_meta_stringdata_QRemoteServiceRegister = {
    {
QT_MOC_LITERAL(0, 0, 22), // "QRemoteServiceRegister"
QT_MOC_LITERAL(1, 23, 18), // "allInstancesClosed"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 14), // "instanceClosed"
QT_MOC_LITERAL(4, 58, 29), // "QRemoteServiceRegister::Entry"
QT_MOC_LITERAL(5, 88, 5), // "entry"
QT_MOC_LITERAL(6, 94, 24), // "quitOnLastInstanceClosed"
QT_MOC_LITERAL(7, 119, 21), // "SecurityAccessOptions"
QT_MOC_LITERAL(8, 141, 20), // "SecurityAccessOption"
QT_MOC_LITERAL(9, 162, 9), // "NoOptions"
QT_MOC_LITERAL(10, 172, 16), // "UserAccessOption"
QT_MOC_LITERAL(11, 189, 17), // "GroupAccessOption"
QT_MOC_LITERAL(12, 207, 17), // "OtherAccessOption"
QT_MOC_LITERAL(13, 225, 17) // "WorldAccessOption"

    },
    "QRemoteServiceRegister\0allInstancesClosed\0"
    "\0instanceClosed\0QRemoteServiceRegister::Entry\0"
    "entry\0quitOnLastInstanceClosed\0"
    "SecurityAccessOptions\0SecurityAccessOption\0"
    "NoOptions\0UserAccessOption\0GroupAccessOption\0"
    "OtherAccessOption\0WorldAccessOption"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QRemoteServiceRegister[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       1,   28, // properties
       1,   31, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x06 /* Public */,
       3,    1,   25,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    5,

 // properties: name, type, flags
       6, QMetaType::Bool, 0x00095103,

 // enums: name, alias, flags, count, data
       7,    8, 0x1,    5,   36,

 // enum data: key, value
       9, uint(QRemoteServiceRegister::NoOptions),
      10, uint(QRemoteServiceRegister::UserAccessOption),
      11, uint(QRemoteServiceRegister::GroupAccessOption),
      12, uint(QRemoteServiceRegister::OtherAccessOption),
      13, uint(QRemoteServiceRegister::WorldAccessOption),

       0        // eod
};

void QRemoteServiceRegister::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QRemoteServiceRegister *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->allInstancesClosed(); break;
        case 1: _t->instanceClosed((*reinterpret_cast< const QRemoteServiceRegister::Entry(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QRemoteServiceRegister::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QRemoteServiceRegister::allInstancesClosed)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QRemoteServiceRegister::*)(const QRemoteServiceRegister::Entry & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QRemoteServiceRegister::instanceClosed)) {
                *result = 1;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QRemoteServiceRegister *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->quitOnLastInstanceClosed(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QRemoteServiceRegister *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setQuitOnLastInstanceClosed(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QRemoteServiceRegister::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QRemoteServiceRegister.data,
    qt_meta_data_QRemoteServiceRegister,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QRemoteServiceRegister::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QRemoteServiceRegister::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QRemoteServiceRegister.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QRemoteServiceRegister::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QRemoteServiceRegister::allInstancesClosed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QRemoteServiceRegister::instanceClosed(const QRemoteServiceRegister::Entry & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
